/*
 * Copyright (c) 2006, Invector Embedded Technologies
 * Copyright (c) 2006, Olimex Ltd
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This file is part of the uIP TCP/IP stack.
 *
 * Author:    Tony Pemberton, Pontus Oldberg, Ivan Vasilev
 *
 */

/*
	This is a demo project for the DM9000-H header board. It uses a specifically-designed
	baseboard with an LPC2294 processor. The ethernet chip is acessed as a memory-mapped
	device with it's two registers at addresses 0x81000004(data) and 0x81000000(registers).
	The test asumes that the three base address select jumpers at the board(SA4-SA6) are connected
	to GND(the factory default)
	This project is based on the following sources:
		The uIP stack
		Invector's DM9000 uIP port for 8051(main and ISR loops and dm9000.c)
		Olimex's LPC-H22xx test code (Low-level routines, all files in the folder Source)		
*/


#include <stdio.h>
#include <string.h>
#include <inarm.h>
#include <iolpc2294.h>
#include "lpc22xx.h"


//#include "flash_lpc.h"
#include "config.h"
#include "main.h"

//#include "system.h"
#define MAC1 0x0013
#define MAC2 0xd4fc
#define MAC3 0xf8e2




#define   UIP_ARP_TIMER   10000
#define   UIP_TX_TIMER    500
#define   LED_TIMER       500

int TX_EventPending;	        // the DM9000 hardware receive event
int ARP_EventPending;	        // trigger the arp timer event
unsigned long half_Sec;
unsigned long ten_Secs;
unsigned long LedTimer;

static volatile int ms_ctr = 0;

extern void eeprom_write(u8_t adress, u16_t data);
extern u16_t eeprom_read(u8_t adress);



void TimerBeat(void) {

  ms_ctr++; // Sleep counter.
  LPC22xxLedSet(2);

  half_Sec--;                           // Decrement half sec counter
  if (half_Sec == 0)                    // Count 1ms intervals for half a second
  {			
    half_Sec = UIP_TX_TIMER;            // and then set the receive poll flag
    TX_EventPending = TRUE;             // the DM9000 hardware receive.
  }

  ten_Secs--;
  if (ten_Secs == 0)                    // Count 1ms intervals for ten seconds
  {
    ten_Secs = UIP_ARP_TIMER;           // and then set the event required to
    ARP_EventPending = TRUE;            // trigger the arp timer if necessary
  }

  LedTimer--;
  if (LedTimer == 0)
  {
    LedTimer = LED_TIMER;
    LPC22xxLedSet(2);
  }

}


void main(void)
{

  unsigned int i;
  u16_t eetmp1, eetmp2, eetmp3;
  LedTimer = LED_TIMER;
  half_Sec = UIP_TX_TIMER;
  ten_Secs = UIP_ARP_TIMER;

  //T0TC = 0;
  // System initialization, this will map the exception vectors.
  LPC22xxSystemInit();

  // Set up peripheral registers.
  LPC22xxInitPIO();

  // First disable interrupts.
  __disable_interrupt();

  // Setup interrupt controller.
  LPC22xxInitVIC();
  LPC22xxInitTimerInterrupt(TimerBeat);

  // Periodic timer initialization.
  LPC22xxInitTimer();

  // Enable interrupts.
  __enable_interrupt();

  // Start periodic timer.
  LPC22xxStartTimer();


  // Initialise the uIP TCP/IP stack.
  uip_init();

  // Initialise the webserver app.
  httpd_init();

  // Initialise the device driver.
  if (InitDM9000())  {

    __disable_interrupt();
    LPC22xxLedSet(0);

    // hang if init failed
    while(1);
  }

  // Initialise the ARP cache.
  uip_arp_init();


  eeprom_write(0, MAC1);
  eeprom_write(1, MAC2);
  eeprom_write(2, MAC3);

  eetmp1 = eeprom_read(0);
  eetmp2 = eeprom_read(1);
  eetmp3 = eeprom_read(2);

  if ((eetmp1!=MAC1)|(eetmp2!=MAC2)|(eetmp3!=MAC3)) {

    __disable_interrupt();
    LPC22xxLedSet(1);

    // hang if init failed
    while(1);
  }

  eetmp1 = eeprom_read(3);


  TX_EventPending = FALSE;	// False to poll the DM9000 receive hardware
  ARP_EventPending = FALSE;	// clear the arp timer event flag

  //EA = TRUE; // Enable interrupts

  while(1)
  {
    // Loops here until either a packet has been received or
    // TX_EventPending (half a sec) to scan for anything to send or
    // ARP_EventPending (10 secs) to send an ARP packet		
    uip_len = DM9000_receive();

    // If uip_len is 0, no packet has been received
    if (uip_len == 0)
    {
      // Test for anything to be sent to 'net
      if (TX_EventPending)
      {
        TX_EventPending = FALSE;  // First clear flag if set

        // UIP_CONNS - nominally 10 - is the maximum simultaneous
        // connections allowed. Scan through all 10
        for (i = 0; i < UIP_CONNS; i++)
        {
          uip_periodic(i);
          // If uip_periodic(i) finds that data that needs to be
          // transmitted to the network, the global variable uip_len
          // will be set to a value > 0.
          if (uip_len > 0)
          {
            /* The uip_arp_out() function should be called when an IP packet should be sent out on the
						Ethernet. This function creates an Ethernet header before the IP header in the uip_buf buffer.
						The Ethernet header will have the correct Ethernet MAC destination address filled in if an
						ARP table entry for the destination IP address (or the IP address of the default router)
						is present. If no such table entry is found, the IP packet is overwritten with an ARP request
						uip_len variable holds the length of the Ethernet frame that should be transmitted. */
            uip_arp_out();
            DM9000_transmit();
          }
        }

#if UIP_UDP
        for (i = 0; i < UIP_UDP_CONNS; i++)
        {
          uip_udp_periodic(i);
          // If the above function invocation resulted in data that
          // should be sent out on the network, the global variable
          // uip_len is set to a value > 0.
          if (uip_len > 0)
          {
            uip_arp_out();
            DM9000_transmit();
          }
        }
#endif /* UIP_UDP */
      }
      // Call the ARP timer function every 10 seconds. Flush dead entries
      if (ARP_EventPending)
      {
        ARP_EventPending = FALSE;
        uip_arp_timer();
      }
    }
    // Packet Received (uip_len != 0) Process incoming
    else
    {
      if (BUF->type == htons(UIP_ETHTYPE_IP))
      {
        // Received an IP packet
        uip_arp_ipin();
        uip_input();
        // If the above function invocation resulted in data that
        // should be sent out on the network, the global variable
        // uip_len is set to a value > 0.
        if (uip_len > 0)
        {
          uip_arp_out();
          DM9000_transmit();
        }
      }
      else
      {
        if (BUF->type == htons(UIP_ETHTYPE_ARP))
        {
          // Received an ARP packet
          uip_arp_arpin();
          // If the above function invocation resulted in data that
          // should be sent out on the network, the global variable
          // uip_len is set to a value > 0.
          if (uip_len > 0)
          {	
            DM9000_transmit();
          }
        }
      }
    }
  }	// end of 'while (1)'	
}




